@@ -4,6 +4,8 @@ from enum      import Enum
 from typing    import List, Optional, Tuple
 from xml.etree import ElementTree as et
 
+from httpx     import AsyncClient
+
 URL = \
     "https://dronebl.org/RPC2"
 HEADERS = {
@@ -44,54 +46,44 @@ class Lookup(object):
         pieces.append(f"comment={self.comment!r}")
         return f"Lookup({', '.join(pieces)})"
 
-class DroneBL(object):
+class BaseDroneBL(object):
     def __init__(self, key: str):
         self._key = key
 
-    def _post(self,
-            method: str
-            ) -> List[et.Element]:
-        data     = ENVELOPE.format(key=self._key, method=method)
-        request  = urllib.request.Request(
-            URL, data.encode("utf8"), HEADERS, method="POST"
-        )
-        response = urllib.request.urlopen(request, timeout=5)
-        decoded  = response.read().decode("utf8")
-        elements = et.fromstring(decoded)
-        return list(elements)
-
-    def add(self,
+    def _make_add(self,
             ip:      str,
             type:    int,
             comment: str,
             port:    Optional[int]=None
-            # return (success, message)
-            ) -> Tuple[bool, str]:
+            ) -> str:
         if port is not None:
             method = M_ADD_PORT
         else:
             method = M_ADD
-
-        method_f = method.format(
+        return method.format(
             ip=ip, type=type, comment=comment, port=port
         )
-        response = self._post(method_f)[0]
+    def _parse_add(self,
+            responses: List[et.Element]
+            ) -> Tuple[bool, str]:
         return (
-            response.tag == "success",
-            response.get("data", "")
+            responses[0].tag == "success",
+            responses[0].get("data", "")
         )
 
-    def lookup(self,
+    def _make_lookup(self,
             ip:   str,
-            type: Optional[str]=None
-            ) -> Optional[Lookup]:
+            type: Optional[int]=None
+            ) -> str:
         if type is not None:
             method = M_LOOKUP_TYPE
         else:
             method = M_LOOKUP
+        return method.format(ip=ip, type=type)
 
-        method_f  = method.format(ip=ip, type=type)
-        responses = self._post(method_f)
+    def _parse_lookup(self,
+            responses: List[et.Element]
+            ) -> Optional[Lookup]:
         if (not responses or
                 not responses[0].tag == "result"):
             return None
@@ -106,12 +98,92 @@ class DroneBL(object):
                 int(response.get("timestamp", ""))
             )
 
+    def _make_remove(self,
+            id: int
+            ) -> str:
+        return M_REMOVE.format(id=id)
+    def _parse_remove(self,
+            responses: List[et.Element]
+            ) -> Tuple[bool, str]:
+        return (
+            responses[0].tag == "success",
+            responses[0].get("data", "")
+        )
+
+class DroneBL(BaseDroneBL):
+    def _post(self,
+            method: str
+            ) -> List[et.Element]:
+        data     = ENVELOPE.format(key=self._key, method=method)
+        request  = urllib.request.Request(
+            URL, data.encode("utf8"), HEADERS, method="POST"
+        )
+        response = urllib.request.urlopen(request, timeout=5)
+        decoded  = response.read().decode("utf8")
+        elements = et.fromstring(decoded)
+        return list(elements)
+
+    def lookup(self,
+            ip:   str,
+            type: Optional[int]=None
+            ) -> Optional[Lookup]:
+        method    = self._make_lookup(ip,type)
+        responses = self._post(method)
+        return self._parse_lookup(responses)
+
+    def add(self,
+            ip:      str,
+            type:    int,
+            comment: str,
+            port:    Optional[int]=None
+            ) -> Tuple[bool, str]:
+        method    = self._make_add(ip, type, comment, port)
+        responses = self._post(method)
+        return self._parse_add(responses)
+
     def remove(self,
             id: int
             ) -> Tuple[bool, str]:
-        method_f = M_REMOVE.format(id=id)
-        response = self._post(method_f)[0]
-        return (
-            response.tag == "success",
-            response.get("data", "")
-        )
+        method    = self._make_remove(id)
+        responses = self._post(method)
+        return self._parse_remove(responses)
+
+class AsyncDroneBL(BaseDroneBL):
+    async def _post(self,
+            method: str
+            ) -> List[et.Element]:
+        data     = ENVELOPE.format(key=self._key, method=method)
+        async with AsyncClient() as client:
+            response = await client.post(
+                URL,
+                data    = data.encode("utf8"),
+                headers = HEADERS
+            )
+        decoded  = response.content.decode("utf8")
+        elements = et.fromstring(decoded)
+        return list(elements)
+
+    async def lookup(self,
+            ip:   str,
+            type: Optional[int]=None
+            ) -> Optional[Lookup]:
+        method    = self._make_lookup(ip,type)
+        responses = await self._post(method)
+        return self._parse_lookup(responses)
+
+    async def add(self,
+            ip:      str,
+            type:    int,
+            comment: str,
+            port:    Optional[int]=None
+            ) -> Tuple[bool, str]:
+        method    = self._make_add(ip, type, comment, port)
+        responses = await self._post(method)
+        return self._parse_add(responses)
+
+    async def remove(self,
+            id: int
+            ) -> Tuple[bool, str]:
+        method    = self._make_remove(id)
+        responses = await self._post(method)
+        return self._parse_remove(responses)
